@@ -3,37 +3,52 @@
 #'Performs an Kruskal-Wallis Test for each assay (by OlinkID) in every panel using stats::kruskal.test.
 #'Performs an Friedman Test for each assay (by OlinkID) in every panel using rstatix::friedman_test.
 #'The function handles factor variable. \cr\cr
-#'Samples that have no variable information or missing factor levels are automatically removed from the analysis (specified in a message if verbose = T).
+#'Samples that have no variable information or missing factor levels are automatically removed from the analysis (specified in a messsage if verbose = TRUE).
 #'Character columns in the input dataframe are automatically converted to factors (specified in a message if verbose = T).
 #'Numerical variables are not converted to factors.
 #'If a numerical variable is to be used as a factor, this conversion needs to be done on the dataframe before the function call. \cr\cr
-#'Inference is specified in a message if verbose = T. \cr
-#'The formula notation of the final model is specified in a message if verbose = T. \cr\cr
+#'Inference is specified in a message if verbose = TRUE. \cr
+#'The formula notation of the final model is specified in a message if verbose = TRUE. \cr\cr
 #'Adjusted p-values are calculated by stats::p.adjust according to the Benjamini & Hochberg (1995) method (“fdr”).
 #'The threshold is determined by logic evaluation of Adjusted_pval < 0.05.
 #'
 #'
 #' @param df NPX or Quantified_value data frame in long format with at least protein name (Assay), OlinkID, UniProt, Panel and a factor with at least 3 levels.
 #' @param variable Single character value.
-#' @param dependence Logical. Default: FALSE. When the groups are independent, the kruskal-Wallis will run, when the groups are dependent, the Friedman test will run.
-#' @param verbose Logical. Default: True. If information about removed samples, factor conversion and final model formula is to be printed to the console.
-#'
+#' @param dependence Boolean. Default: FALSE. When the groups are independent, the kruskal-Wallis will run, when the groups are dependent, the Friedman test will run.
+#' @param subject Group information for the repeated measurement. If (dependence = TRUE), this parameter need to be specified.
+#' @param verbose Boolean. Default: True. If information about removed samples, factor conversion and final model formula is to be printed to the console.
 #' @return A tibble containing the Kruskal-Wallis Test or Friedman Test results for every protein.
-#' The tibble is arranged by ascending p-values.
+#'
+#' Columns include:
+#' \itemize{
+#'  \item{Assay:} "character" Protein symbol
+#'  \item{OlinkID:} "character" Olink specific ID
+#'  \item{UniProt:} "character" Olink specific ID
+#'  \item{Panel:} "character" Name of Olink Panel
+#'  \item{term:} "character" term in model
+#'  \item{df:} "numeric" degrees of freedom
+#'  \item{method:} "character" which method was used
+#'  \item{statistic:} "named numeric" the value of the test statistic with a name describing it
+#'  \item{p.value:} "numeric" p-value for the test
+#'  \item{Adjusted_pval:} "numeric" adjusted p-value for the test (Benjamini&Hochberg)
+#'  \item{Threshold:} "character" if adjusted p-value is significant or not (< 0.05)
+#' }
+#'
 #' @export
 #' @examples \donttest{
 #'
 #' library(dplyr)
 #'
-#' npx_df <- npx_data1 %>% filter(!grepl('control', SampleID, ignore.case = TRUE))
+#' # One-way Kruskal-Wallis Test
+#' kruskal_results <- olink_one_non_parametric(df = npx_data1,
+#'                                             variable = "Site")
 #'
-#' #One-way Kruskal-Wallis Test.
-#' #Results in a model NPX~Time
-#' Kruskal_results <- olink_one_non_parametric(df = npx_df, variable = "Time")
-#'
-#' #One-way Friedman Test.
-#' #Results in a model NPX~Time
-#' Friedman_results <- olink_one_non_parametric(df = npx_df, variable = "Time", dependence = TRUE)}
+#' #Friedman Test
+#' friedman_results <- olink_one_non_parametric(df = npx_data1,
+#'                                              variable = "Time",
+#'                                              subject = "Subject",
+#'                                              dependence = TRUE)}
 #'
 #' @importFrom dplyr n filter group_by summarise ungroup pull n_distinct do select arrange mutate
 #' @importFrom stringr str_detect
@@ -45,19 +60,18 @@
 olink_one_non_parametric <- function(df,
                                      variable,
                                      dependence = FALSE,
-                                     verbose=T
-){
-
+                                     subject = NULL,
+                                     verbose=TRUE
+                                     ){
   if(missing(df) | missing(variable)){
     stop('The df and variable arguments need to be specified.')
   }
 
   withCallingHandlers({
-
     #Filtering on valid OlinkID
     df <- df %>%
       dplyr::filter(stringr::str_detect(OlinkID,
-                                 "OID[0-9]{5}"))
+                                        "OID[0-9]{5}"))
 
     #Variables to check
     variable_testers <- intersect(c(variable), names(df))
@@ -73,32 +87,9 @@ olink_one_non_parametric <- function(df,
 
     df <- df[!is.na(df[[variable_testers]]),]
 
-    # # Check whether it is NPX or QUANT
-
-    if ('NPX' %in% colnames(df)) {
-      data_type <- 'NPX'
-    } else if ('Quantified_value' %in% colnames(df)) {
-      data_type <- 'Quantified_value'
-    } else {
-      stop('The NPX or Quantified_value is not in the df.')}
-
-    #Not testing assays that have all NA:s
-    all_nas <- df  %>%
-      dplyr::group_by(OlinkID) %>%
-      dplyr::summarise(n = dplyr::n(), n_na = sum(is.na(!!rlang::ensym(data_type)))) %>%
-      dplyr::ungroup() %>%
-      dplyr::filter(n == n_na) %>%
-      dplyr::pull(OlinkID)
-
-
-    if(length(all_nas) > 0) {
-
-      warning(paste0('The assays ',
-                     paste(all_nas, collapse = ', '),
-                     ' have only NA:s. They will not be tested.'),
-              call. = F)
-
-    }
+    #Check data format
+    npxCheck <- npxCheck(df)
+    data_type <- npxCheck$data_type #Temporary fix to avoid issues with rlang::ensym downstream
 
     ##Convert character vars to factor
     converted.vars <- NULL
@@ -112,21 +103,17 @@ olink_one_non_parametric <- function(df,
       }
     }
 
-
     #Not testing assays that have all NA:s in one level
     #Every sample needs to have a unique level of the factor
 
     nas_in_var <- character(0)
 
-
     single_fixed_effects <- variable
 
-
-
     for(effect in single_fixed_effects){
 
       current_nas <- df %>%
-        dplyr::filter(!(OlinkID %in% all_nas)) %>%
+        dplyr::filter(!(OlinkID %in% npxCheck$all_nas)) %>%
         dplyr::group_by(OlinkID, !!rlang::ensym(effect)) %>%
         dplyr::summarise(n = dplyr::n(), n_na = sum(is.na(!!rlang::ensym(data_type))),.groups="drop") %>%
         dplyr::filter(n == n_na) %>%
@@ -134,9 +121,7 @@ olink_one_non_parametric <- function(df,
         dplyr::pull(OlinkID)
 
       if(length(current_nas) > 0) {
-
         nas_in_var <- c(nas_in_var, current_nas)
-
         warning(paste0('The assay(s) ',
                        current_nas,
                        ' has only NA:s in atleast one level of ',
@@ -158,18 +143,14 @@ olink_one_non_parametric <- function(df,
                     effect,
                     ". Only one level per sample is allowed."))
       }
-
-
     }
 
     formula_string <- paste0(data_type, "~", paste(variable,collapse="*"))
 
-
     #Get factors
     fact.vars <- sapply(variable_testers, function(x) is.factor(df[[x]]))
     fact.vars <- names(fact.vars)[fact.vars]
 
-
     #Print verbose message
     if(verbose){
       if(!is.null(removed.sampleids) & length(removed.sampleids) >0){
@@ -186,20 +167,52 @@ olink_one_non_parametric <- function(df,
       }
     }
 
-
     if (dependence){
+      if (is.null(subject)){
+        stop("Subject variable need to be specified!")
+      }
       if(verbose){message(paste("Friedman model fit to each assay: "),formula_string)}
-      formula_string <- paste0(formula_string,"|","group_id")
-      p.val <- df %>%
-        dplyr::filter(!(OlinkID %in% all_nas)) %>%
-        dplyr::filter(!(OlinkID %in% nas_in_var)) %>%
-        dplyr::group_by(Assay,OlinkID, UniProt, Panel,!!!rlang::syms(variable)) %>%
-        dplyr::mutate(group_id = as.factor (1:dplyr::n()))%>%
+
+      formula_string <- paste0(formula_string,"|",subject)
+      # add repeat measurement groups
+      df_nas_remove <- df %>%
+        dplyr::filter(!(OlinkID %in% npxCheck$all_nas)) %>%
+        dplyr::filter(!(OlinkID %in% nas_in_var))
+      # remove subject without complete data
+      ## number of the items in the subject
+
+      n_subject <- df_nas_remove %>%
+        dplyr::select(!!!rlang::syms(variable)) %>%
+        stats::na.omit() %>%
+        unique() %>%
+        pull() %>%
+        length()
+
+      subject_remove <- df_nas_remove %>%
+        dplyr::filter(!is.na(NPX)) %>%
+        dplyr::group_by(Assay, !!!rlang::syms(subject)) %>%
+        dplyr::summarize(n = n(),.groups = "drop") %>%
+        dplyr::filter(n == n_subject) %>%
+        dplyr::mutate(Friedman_remove = "no") %>%
+        dplyr::select(-n)
+
+      if(length(subject_remove %>%
+                dplyr::select(!!!rlang::syms(subject)) %>%
+                dplyr::pull() %>%
+                unique()) < length(df_nas_remove %>%
+                                   dplyr::select(!!!rlang::syms(subject)) %>%
+                                   dplyr::pull() %>%
+                                   unique())){
+        message(paste("Subjects removed due to incomplete data"))
+      }
+
+      p.val <- df_nas_remove %>%
+        dplyr::left_join(subject_remove, by = c("Assay",subject)) %>%
+        dplyr::filter(Friedman_remove == "no") %>%
         rstatix::convert_as_factor(!!!rlang::syms(variable)) %>%
-        dplyr::ungroup(!!!rlang::syms(variable))%>%
         dplyr::group_by(Assay, OlinkID, UniProt, Panel) %>%
         dplyr::do(rstatix::friedman_test(as.formula(formula_string),
-                                   data=.,na.action=na.omit)) %>%
+                                         data=.,na.action=na.omit)) %>%
         dplyr::ungroup() %>%
         dplyr::mutate(Adjusted_pval = p.adjust(p, method = "BH")) %>%
         dplyr::mutate(Threshold  = ifelse(Adjusted_pval<0.05, "Significant","Non-significant")) %>%
@@ -209,16 +222,14 @@ olink_one_non_parametric <- function(df,
         dplyr::mutate(term = variable) %>%
         dplyr::rename(p.value = p) %>%
         dplyr::select(Assay, OlinkID, UniProt, Panel, term, df, method, statistic, p.value, Adjusted_pval, Threshold)
-
-
- }else{
-   if(verbose){message(paste("Kruskal model fit to each assay: "),formula_string)}
+    }else{
+      if(verbose){message(paste("Kruskal model fit to each assay: "),formula_string)}
       p.val <- df %>%
-        dplyr::filter(!(OlinkID %in% all_nas)) %>%
+        dplyr::filter(!(OlinkID %in% npxCheck$all_nas)) %>%
         dplyr::filter(!(OlinkID %in% nas_in_var)) %>%
         dplyr::group_by(Assay, OlinkID, UniProt, Panel) %>%
         dplyr::do(broom::tidy(kruskal.test(as.formula(formula_string),
-                              data=.))) %>%
+                                           data=.))) %>%
         dplyr::ungroup() %>%
         dplyr::mutate(Adjusted_pval=p.adjust(p.value,method="BH")) %>%
         dplyr::mutate(Threshold  = ifelse(Adjusted_pval<0.05,"Significant","Non-significant")) %>%
@@ -227,7 +238,6 @@ olink_one_non_parametric <- function(df,
         dplyr::mutate(term = variable) %>%
         dplyr::rename(df = parameter) %>%
         dplyr::select(Assay, OlinkID, UniProt, Panel, term, df, method, statistic, p.value, Adjusted_pval, Threshold)
-
   }, warning = function(w) {
     if (grepl(x = w, pattern = glob2rx("*contains implicit NA, consider using*")))
       invokeRestart("muffleWarning")
@@ -237,67 +247,88 @@ olink_one_non_parametric <- function(df,
 
 
 
-#'Function which performs a Wilcox posthoc test per protein.
+#'Function which performs posthoc test per protein for the results from Friedman or Kruskal-Wallis Test.
 #'
-#'Performs a posthoc test using rstatix::wilcox_test with Benjamini-Hochberg p-value adjustment per assay (by OlinkID) for each panel at confidence level 0.95.
-#'See \code{olink_kruskal} for details of input notation. \cr\cr
+#'Performs a posthoc test using rstatix::wilcox_test or FSA::dennTest with Benjamini-Hochberg p-value adjustment per assay (by OlinkID) for each panel at confidence level 0.95.
+#'See \code{olink_one_non_parametric} for details of input notation. \cr\cr
 #'The function handles both factor and numerical variables.
-#'The posthoc test for a numerical variable compares the difference in medians of the outcome variable (default: NPX) for 1 standard deviation difference in the numerical variable, e.g.
-#'median NPX at mean(numerical variable) versus median NPX at median(numerical variable) + 1*SD(numerical variable).
 #'
 #' @param df NPX data frame in long format with at least protein name (Assay), OlinkID, UniProt, Panel and a factor with at least 3 levels.
 #' @param olinkid_list Character vector of OlinkID's on which to perform post hoc analysis. If not specified, all assays in df are used.
 #' @param variable Single character value or character array.
-#' @param verbose Logical. Default: True. If information about removed samples, factor conversion and final model formula is to be printed to the console.
+#' @param test Single character value indicates running the post hoc test for friedman or kruskal.
+#' @param verbose Boolean. Deafult: True. If information about removed samples, factor conversion and final model formula is to be printed to the console.
 #' @return Tibble of posthoc tests for specified effect, arranged by ascending adjusted p-values.
+#'
+#' Columns include:
+#' \itemize{
+#'  \item{Assay:} "character" Protein symbol
+#'  \item{OlinkID:} "character" Olink specific ID
+#'  \item{UniProt:} "character" Olink specific ID
+#'  \item{Panel:} "character" Name of Olink Panel
+#'  \item{term:} "character" term in model
+#'  \item{contrast:} "character" the groups that were compared
+#'  \item{estimate:} "numeric" the value of the test statistic with a name describing it
+#'  \item{Adjusted_pval:} "numeric" adjusted p-value for the test
+#'  \item{Threshold:} "character" if adjusted p-value is significant or not (< 0.05)
+#' }
+#'
 #' @export
 #' @examples \donttest{
-#'
 #' library(dplyr)
 #'
-#' npx_df <- npx_data1 %>% filter(!grepl('control',SampleID, ignore.case = TRUE))
-#'
-#' #Kruskal-Wallis Test
-#' kruskal_wallis_results <- olink_one_non_parametric(npx_df, "Site")
+#' # One-way Kruskal-Wallis Test
+#' kruskal_results <- olink_one_non_parametric(df = npx_data1,
+#'                                             variable = "Site")
 #'
 #' #Friedman Test
-#' Friedman_results <- olink_one_non_parametric(npx_df, "Time", dependence = TRUE)
+#' friedman_results <- olink_one_non_parametric(df = npx_data1,
+#'                                             variable = "Time",
+#'                                             subject = "Subject",
+#'                                             dependence = TRUE)
 #'
 #' #Posthoc test for the results from Friedman Test
-#' #Filtering out significant and relevant results.
-#' significant_assays <- Friedman_results %>%
-#' filter(Threshold == 'Significant') %>%
-#' dplyr::select(OlinkID) %>%
-#' distinct() %>%
-#' pull()
-#'
-#' #Posthoc
-#' friedman_posthoc_results <- olink_one_non_parametric_posthoc(npx_df, variable = c("Time"),
-#' olinkid_list = significant_assays)}
+#'friedman_posthoc_results <- olink_one_non_parametric_posthoc(npx_data1,
+#'                                                             variable = "Time",
+#'                                                             test = "friedman",
+#'                                                             olinkid_list = {friedman_results %>%
+#'                                                               filter(Threshold == 'Significant') %>%
+#'                                                               dplyr::select(OlinkID) %>%
+#'                                                               distinct() %>%
+#'                                                               pull()})}
 #'
 #' @importFrom dplyr filter group_by ungroup pull do select arrange mutate
 #' @importFrom stringr str_detect
 #' @importFrom rstatix wilcox_test
 
 
+
 olink_one_non_parametric_posthoc <- function(df,
                                              olinkid_list = NULL,
                                              variable,
-                                             verbose=T
+                                             test = "kruskal",
+                                             verbose=TRUE
                                              ){
+  if (test != "friedman"){
+    if(!requireNamespace("FSA", quietly = TRUE)){
+      stop("Kruskal test requires FSA package.
+         Please install FSA before continuing.
 
-
-
+         install.packages(\"FSA\")")
+    }
+  }
   if(missing(df) | missing(variable)){
-    stop('The df and variable and effect arguments need to be specified.')
+    stop('The df and variable arguments need to be specified.')
+  }
+  if (!(test %in% c("friedman","kruskal"))){
+    stop('The type of the test need to be specified as kruskal or friedman')
   }
 
   withCallingHandlers({
 
-    #Filtering on valid OlinkID
+    # Filtering on valid OlinkID
     df <- df %>%
-      dplyr::filter(stringr::str_detect(OlinkID,
-                                 "OID[0-9]{5}"))
+      dplyr::filter(stringr::str_detect(OlinkID, "OID[0-9]{5}"))
 
     if(is.null(olinkid_list)){
       olinkid_list <- df %>%
@@ -306,23 +337,14 @@ olink_one_non_parametric_posthoc <- function(df,
         dplyr::pull()
     }
 
-    # # Check whether it is NPX or QUANT
-
-    if ('NPX' %in% colnames(df)) {
-      data_type <- 'NPX'
-    } else if ('Quantified_value' %in% colnames(df)) {
-      data_type <- 'Quantified_value'
-    } else {
-      stop('The NPX or Quantified_value is not in the df.')}
-
+    #Check data format
+    npxCheck <- npxCheck(df)
+    data_type <- npxCheck$data_type #Temporary fix to avoid issues with rlang::ensym downstream
 
     variable_testers <- intersect(c(variable), names(df))
     ##Remove rows where variables or covariate is NA (cant include in analysis anyway)
     removed.sampleids <- NULL
 
-    #removed.sampleids <- unique(c(removed.sampleids,df$SampleID[is.na(df[[variable_testers]])]))
-    #df <- df[!is.na(df[[variable_testers]]),]
-
     for(i in variable_testers){
       removed.sampleids <- unique(c(removed.sampleids,df$SampleID[is.na(df[[i]])]))
       df <- df[!is.na(df[[i]]),]
@@ -333,11 +355,11 @@ olink_one_non_parametric_posthoc <- function(df,
     num.vars <- NULL
 
     if(is.character(df[[variable_testers]])){
-        df[[variable_testers]] <- factor(df[[variable_testers]])
-        converted.vars <- c(converted.vars,variable_testers)
-        } else if(is.numeric(df[[variable_testers]])){
-        num.vars <- c(num.vars,variable_testers)
-        }
+      df[[variable_testers]] <- factor(df[[variable_testers]])
+      converted.vars <- c(converted.vars,variable_testers)
+    } else if(is.numeric(df[[variable_testers]])){
+      num.vars <- c(num.vars,variable_testers)
+    }
 
     formula_string <- paste0(data_type, "~", paste(variable,collapse="*"))
 
@@ -355,28 +377,40 @@ olink_one_non_parametric_posthoc <- function(df,
         message(paste0("Variables treated as numeric: ",
                        paste(num.vars,collapse = ", ")))
       }
-      if(any(variable %in% num.vars)){
-        message(paste0("Numeric variables post-hoc performed using Mean and Mean + 1SD: ",
-                       paste(num.vars[num.vars%in%variable],collapse = ", ")))
-      }
-      message(paste("Means estimated for each assay from non-parametric model: ",formula_string))
     }
 
-
-
-    p.hoc_val <- df %>%
-      dplyr::filter(OlinkID %in% olinkid_list) %>%
-      dplyr::mutate(OlinkID = factor(OlinkID, levels = olinkid_list)) %>%
-      dplyr::group_by(Assay, OlinkID, UniProt, Panel) %>%
-      dplyr::do(rstatix::wilcox_test(data =., as.formula(formula_string), p.adjust.method = "BH",detailed = TRUE, conf.level = 0.95)) %>%
-      dplyr::ungroup() %>%
-      dplyr::mutate("variable" = variable) %>%
-      dplyr::mutate(Threshold = if_else(p.adj < 0.05,'Significant','Non-significant')) %>%
-      dplyr::rename(term = variable) %>%
-      dplyr::mutate(contrast = paste(group1, group2, sep = " - ")) %>%
-      dplyr::rename(Adjusted_pval = p.adj) %>%
-      dplyr::select(all_of(c("Assay", "OlinkID", "UniProt", "Panel", "term", "contrast", "estimate","conf.low", "conf.high", "Adjusted_pval","Threshold")))
-
+    if(test == "friedman"){
+      message("Pairwise comparisons for Friedman test using paired Wilcoxon signed-rank test were performed")
+      p.hoc_val <- df %>%
+        dplyr::filter(OlinkID %in% olinkid_list) %>%
+        dplyr::filter(!(OlinkID %in% npxCheck$all_nas)) %>% #Exclude assays where all samples have NPX=NA
+        dplyr::mutate(OlinkID = factor(OlinkID, levels = olinkid_list)) %>%
+        dplyr::group_by(Assay, OlinkID, UniProt, Panel) %>%
+        dplyr::do(rstatix::wilcox_test(data =., as.formula(formula_string), p.adjust.method = "BH",detailed = TRUE, conf.level = 0.95,paired = T)) %>%
+        dplyr::ungroup() %>%
+        dplyr::mutate("variable" = variable) %>%
+        dplyr::mutate(Threshold = if_else(p.adj < 0.05,'Significant','Non-significant')) %>%
+        dplyr::rename(term = variable) %>%
+        dplyr::mutate(contrast = paste(group1, group2, sep = " - ")) %>%
+        dplyr::rename(Adjusted_pval = p.adj) %>%
+        dplyr::select(all_of(c("Assay", "OlinkID", "UniProt", "Panel", "term", "contrast", "estimate", "Adjusted_pval", "Threshold")))
+    } else {
+      message("Pairwise comparisons for Kruskal-Wallis test using Dunn test were performed")
+      p.hoc_val <- df %>%
+        dplyr::filter(OlinkID %in% olinkid_list) %>%
+        dplyr::filter(!(OlinkID %in% npxCheck$all_nas)) %>% #Exclude assays where all samples have NPX=NA
+        dplyr::mutate(OlinkID = factor(OlinkID, levels = olinkid_list)) %>%
+        dplyr::group_by(Assay, OlinkID, UniProt, Panel) %>%
+        dplyr::do(FSA::dunnTest(data =., as.formula(formula_string), method = "bh")$res) %>%
+        dplyr::ungroup() %>%
+        dplyr::mutate("variable" = variable) %>%
+        dplyr::mutate(Threshold = if_else(P.adj < 0.05,'Significant','Non-significant')) %>%
+        dplyr::rename(term = variable) %>%
+        dplyr::rename(contrast = Comparison) %>%
+        dplyr::rename(Adjusted_pval = P.adj) %>%
+        dplyr::rename(estimate = Z) %>%
+        dplyr::select(all_of(c("Assay", "OlinkID", "UniProt", "Panel", "term", "contrast", "estimate", "Adjusted_pval", "Threshold")))
+    }
 
     return(p.hoc_val)
 
